unit CopyMoveForm;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls, Buttons, MainForm, SearchDir, finderpopup;

type
  TFrmCopyMove = class(TForm)
    BvlLine: TBevel;
    BtnOK: TButton;
    BtnCancel: TButton;
    Label1: TLabel;
    EdtTarget: TEdit;
    SpdBrowse: TSpeedButton;
    GrpOptions: TGroupBox;
    ChkRecursive: TCheckBox;
    ChkUseFilter: TCheckBox;
    Bevel1: TBevel;
    Label2: TLabel;
    Label3: TLabel;
    Label4: TLabel;
    LblSize: TLabel;
    LblSubDirs: TLabel;
    LblFileNum: TLabel;
    ImgCopyMove: TImage;
    ImgCopy: TImage;
    ImgMove: TImage;
    BtnHelp: TButton;
    ChkDontRemoveEmptyDirectories: TCheckBox;
    ChkCopyOnlyNewFiles: TCheckBox;
    ChkAlwaysOverwriteOldFiles: TCheckBox;
    procedure FormShow(Sender: TObject);
    procedure ChkRecursiveClick(Sender: TObject);
    procedure SpdBrowseClick(Sender: TObject);
    procedure ChkUseFilterClick(Sender: TObject);
    procedure BtnHelpClick(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
  private
    { Private-Deklarationen }
    Procedure WMNCHitTest(Var Msg: TMessage); Message WM_NCHITTEST;
  public
    { Public-Deklarationen }
    SourceList: Integer;
    dirInfoThread: TDirInfoThread;
    Procedure UpdateInfo;
//    TDirInfoCallback = Procedure(FileNum, DirNum: Integer; SizeInBytes: Int64; SysFiles, HiddenFiles, ReadOnlyFiles, ArchiveFiles: Longint);
  end;

var
  FrmCopyMove: TFrmCopyMove;

implementation

{$R *.DFM}

Procedure dirInfoThreadCallback(FileNum, DirNum: Integer; SizeInBytes: Int64; SysFiles, HiddenFiles, ReadOnlyFiles, ArchiveFiles: Longint; LastCall: boolean);
var
   siz: string;
begin
     if LastCall then begin
        DIThreadFinished := TRUE;
        DISizeInBytes := sizeInBytes;
        DIFileNum := fileNum;
        DISubDirs := dirNum;
        DIObjNum := fileNum + dirNum;
        DISysFiles := sysFiles;
        DIHiddenFiles := hiddenFiles;
        DIReadOnlyFiles := ReadOnlyFiles;
        DIArchiveFiles := ArchiveFiles;
     end;
     FrmCopyMove.LblSubDirs.Caption:=Formatted(DirNum);
     FrmCopyMove.LblFileNum.Caption:=Formatted(FileNum);
      If SizeInBytes >= 1024*1024*1024 Then Begin
        Str(SizeInBytes/(1024*1024*1024):5:1,Siz);
        Siz:=Siz+' GB ('+Formatted(SizeInBytes)+' Bytes)';
     End Else If SizeInBytes >= 1024*1024 Then Begin
        Str(SizeInBytes/(1024*1024):3:1,Siz);
        Siz:=Siz+' MB ('+Formatted(SizeInBytes)+' Bytes)';
     End Else If SizeInBytes >= 1024 Then Begin
        Str(SizeInBytes/1024:3:1,Siz);
        Siz:=Siz+' KB ('+Formatted(SizeInBytes)+' Bytes)';
     End Else Begin
        Siz:=Formatted(SizeInBytes)+' Bytes';
     End;
     FrmCopyMove.LblSize.Caption:=Siz;
end;

Procedure TFrmCopyMove.UpdateInfo;
var
   tmp: TFilterRec;
begin
     if assigned(dirInfoThread) then begin
        dirInfoThread.Terminate;
        dirInfoThread.WaitFor;
        dirInfoThread.Destroy;
        dirInfoThread := NIL;
     end;
     if ChkUseFilter.Checked then tmp := data[currentList].Filter else tmp := FilterAllFiles;
     dirInfoThread := TDirInfoThread.Create(currentList, ChkRecursive.Checked, FALSE, tmp, 250, dirInfoThreadCallback);
End;

procedure TFrmCopyMove.FormShow(Sender: TObject);
begin
     EdtTarget.SelectAll;
     EdtTarget.SetFocus;
     ChkRecursiveClick(Nil);
     updateInfo;
end;

Procedure UpdateChkDontRemoveEmptyDirectoriesEnabled;
Begin
     With FrmCopyMove Do
          ChkDontRemoveEmptyDirectories.Enabled:=(ChkRecursive.Checked And ChkUseFilter.Checked And (cmAction = cmMove));
End;

procedure TFrmCopyMove.ChkRecursiveClick(Sender: TObject);
begin
     ChkUseFilter.Enabled:=ChkRecursive.Checked;
     UpdateChkDontRemoveEmptyDirectoriesEnabled;
     UpdateInfo;
end;

function showFinder(sender: TSpeedButton; stdPath: string): string;
var
   pt: TPoint;
   scr: TPoint;
begin
     pt := point(sender.width, sender.height);
     scr := sender.ClientToScreen(pt);
     result := popupFinder(scr.x, scr.y, FINDER_ALL, FrmCopyMove.Handle, true, stdPath);
     if result = '' then result := stdPath;
end;

procedure TFrmCopyMove.SpdBrowseClick(Sender: TObject);
begin
     EdtTarget.Text := showFinder(SpdBrowse, edtTarget.Text);
     EdtTarget.SelectAll;
end;

procedure TFrmCopyMove.ChkUseFilterClick(Sender: TObject);
begin
     UpdateChkDontRemoveEmptyDirectoriesEnabled;
     UpdateInfo;
end;

procedure TFrmCopyMove.BtnHelpClick(Sender: TObject);
begin
     Application.HelpCommand(HELP_CONTEXT, HelpContext);
end;

Procedure TFrmCopyMove.WMNCHitTest(Var Msg : TMessage);
Var
   i: Integer;
   r: TRect;
   ctrl: TControl;

Function LeftMouseButtonPressed: Boolean;
Begin
     If GetSystemMetrics(SM_SWAPBUTTON) = 0 Then
        Result:=(GetAsyncKeyState(VK_LBUTTON) < 0)
     Else
        Result:=(GetAsyncKeyState(VK_RBUTTON) < 0);
End;

Begin
     If Options.WindowClientDraggable And LeftMouseButtonPressed Then Begin
        If (Msg.LParamLo >= Self.ClientToScreen(Point(0, 0)).x) And
           (Msg.LParamHi >= Self.ClientToScreen(Point(0, 0)).y) And
           (Msg.LParamLo <= Self.ClientToScreen(Point(Self.ClientWidth, Self.ClientHeight)).x) And
           (Msg.LParamHi <= Self.ClientToScreen(Point(Self.ClientWidth, Self.ClientHeight)).y)
        Then Begin
             For i:=0 To Self.ComponentCount-1 Do Begin
                 Try
                    If Self.Components[i] Is TControl Then Begin
                       ctrl:=TControl(Self.Components[i]);
                       r:=ctrl.BoundsRect;
                       If (Msg.LParamLo >= Self.ClientToScreen(Point(r.Left, r.Top)).x) And
                          (Msg.LParamHi >= Self.ClientToScreen(Point(r.Left, r.Top)).y) And
                          (Msg.LParamLo <= Self.ClientToScreen(Point(r.Right, r.Bottom)).x) And
                          (Msg.LParamHi <= Self.ClientToScreen(Point(r.Right, r.Bottom)).y) And ctrl.Visible
                       Then Begin
                          Msg.Result:=DefWindowProc(Handle, Msg.Msg, Msg.WParam, Msg.LParam);
                          Exit;
                       End;
                    End;
                 Except
                    Msg.Result:=DefWindowProc(Handle, Msg.Msg, Msg.WParam, Msg.LParam);
                    Exit;
                 End;
             End;
             Msg.Result:=HTCAPTION
        End Else
           Msg.Result:=DefWindowProc(Handle, Msg.Msg, Msg.WParam, Msg.LParam);
     End Else
        Msg.Result:=DefWindowProc(Handle, Msg.Msg, Msg.WParam, Msg.LParam);
End;

procedure TFrmCopyMove.FormClose(Sender: TObject;
  var Action: TCloseAction);
begin
     if assigned(dirInfoThread) then begin
        dirInfoThread.Terminate;
        dirInfoThread.WaitFor;
        dirInfoThread.Destroy;
        dirInfoThread := NIL;
     end;
end;

end.
