Unit cooldir;

Interface

Uses
    Messages, Windows, SysUtils, Classes, Controls, Forms, Menus, Graphics, StdCtrls, ExtCtrls, shlobj, shellapi, activex;

Type
  TPathItem = record
     folderName: string;
     Width, Height, left, top: integer;
     hilite: boolean;
     active: boolean;
     staticText: boolean;
     grayed: boolean;
  end;

  TQueryChangePathEvent = procedure(sender: TObject; path: string) of object;

  TCoolDirectoryBar = class;

  TCoolDirectoryBar = class(TCustomControl)
  Private
    offset: integer;
    scrollLeftVisible, scrollRightVisible: boolean;
    scrollLeftHilite, ScrollRightHilite: boolean;
    fDown: Boolean;
    fPath, fOldPath: string;
    pathItems: array[0..MAX_PATH] of TPathItem;
    pathItemsCount: integer;
    pathWidth: integer;
    fOnQueryChangePath: TQueryChangePathEvent;
    hilited: boolean;
    scrollTimer: TTimer;
    timerScrollDirection: integer;
    scrollTimerActive: boolean;
    Procedure SetDown(Value: Boolean);
    procedure CMMouseEnter(var Message: TMessage); message CM_MOUSEENTER;
    procedure WMMouseMove(var Message: TMessage); message WM_MOUSEMOVE;
    procedure CMMouseLeave(var Message: TMessage); message CM_MOUSELEAVE;
    function GetPath: string;
    procedure setPath(Path: string);
    procedure queryChangePath(index: integer);
    function mouseOverScrollBtn(x, y: integer): integer;
    procedure scrollToItem(index: integer; repaint: boolean);
    procedure timerInterval(sender: TObject);
    procedure killTimer;
  Public
    constructor Create(AOwner: TComponent); override;
    procedure paint; override;
    procedure MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
    procedure MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
    procedure Resize; override;
  published
    property Align;
    property AutoSize;
    property Color;
    property DragCursor;
    property DragMode;
    property Enabled;
    property Font;
    property ParentColor;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property ShowHint;
    property TabOrder;
    property TabStop;
    property Visible;
    property OnClick;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnStartDrag;
    Property Down: Boolean Read fDown Write SetDown;
    Property Path: string Read GetPath Write SetPath;
    Property OnQueryChangePath: TQueryChangePathEvent read fOnQueryChangePath write fOnQueryChangePath;
  end;

Implementation

const
     scrollerWidth = 12;
     LEFT_SCROLLER = 0;
     RIGHT_SCROLLER = 1;
     NO_SCROLLER = -1;
     SCROLLER_INTERVAL = 200;
     OFFSET_INCREMENT = 50;

procedure TCoolDirectoryBar.killTimer;
begin
     if assigned(scrollTimer) then begin
        scrollTimer.Free;
        scrollTimer := NIL;
        timerScrollDirection := NO_SCROLLER;
     end;
end;

procedure TCoolDirectoryBar.timerInterval;
begin
     if timerScrollDirection = LEFT_SCROLLER then begin
        dec(offset, OFFSET_INCREMENT);
        if offset < 0 then offset := 0;
        Paint;
     end else if timerScrollDirection = RIGHT_SCROLLER then begin
        inc(offset, OFFSET_INCREMENT);
        if (pathWidth - offset) < (self.Width - 2) then offset := (pathWidth - self.Width);
        Paint;
     end;
     scrollTimerActive := TRUE;
end;

procedure TCoolDirectoryBar.Resize;
var
   i: integer;
begin
     if pathItemsCount > 0 then begin
        for i := 0 to pathItemsCount - 1 do begin
            if pathItems[i].active then scrollToItem(i + 1, false);
        end;
     end;
     inherited;
end;

function TCoolDirectoryBar.mouseOverScrollBtn(x, y: integer): integer;
begin
     result := NO_SCROLLER;
     if scrollLeftVisible then begin
        if (x > 1) and (x < (1 + scrollerWidth)) then result := LEFT_SCROLLER;
     end;
     if scrollRightVisible then begin
       if (x > (self.Width - 1 - scrollerWidth)) and (x < (self.Width - 1)) then result := RIGHT_SCROLLER;
     end;
end;

procedure TCoolDirectoryBar.scrollToItem;
var
   tmpWidth: integer;
begin
     if (index > 0) and (index < pathItemsCount) then begin
        if pathWidth <= (self.Width - 2) then begin
           offset := 0;
        end else begin
           tmpWidth := pathItems[index].left + pathItems[index].Width;
           if tmpWidth <= (self.Width - 2) then begin
              offset := 0;
           end else begin
              offset := (tmpWidth - self.Width + 2);
           end;
        end;
     end;
     if repaint then paint;
end;

procedure TCoolDirectoryBar.MouseDown;
var
   res: integer;
begin
     if button = mbLeft then begin
        res := mouseOverScrollBtn(x, y);
        if res <> NO_SCROLLER then begin
           if not assigned(scrollTimer) then begin
              scrollTimer := TTimer.Create(self);
              scrollTimer.Interval := SCROLLER_INTERVAL;
              scrollTimer.OnTimer := timerInterval;
              scrollTimerActive := FALSE;
           end;
           timerScrollDirection := res;
        end;
     end;
     down := TRUE;
     hilited := FALSE;
     paint;
     inherited;
end;

procedure TCoolDirectoryBar.MouseUp;
var
   i: integer;
   res: integer;
begin
     if button = mbLeft then begin
        res := mouseOverScrollBtn(x, y);
        if (res <> NO_SCROLLER) and (not scrollTimerActive) then begin
           if res = LEFT_SCROLLER then begin
              dec(offset, OFFSET_INCREMENT);
              if offset < 0 then offset := 0;
              Paint;
           end else if res = RIGHT_SCROLLER then begin
              inc(offset, OFFSET_INCREMENT);
              if (pathWidth - offset) < (self.Width - 2) then offset := (pathWidth - self.Width);
              Paint;
           end;
        end else if pathItemsCount > 0 then begin
           if not assigned(scrollTimer) then begin
              for i:=0 to pathItemsCount - 1 do begin
                  if not pathItems[i].staticText then begin
                     if ((pathItems[i].left - offset) <= x) and (x < (pathItems[i].left + pathItems[i].width - offset)) then begin
                        if (not pathItems[i].active) then queryChangePath(i);
                     end;
                  end;
              end;
              hilited := FALSE;
           end;
        end;
        killTimer;
     end;
end;

Constructor TCoolDirectoryBar.Create;
Begin
     Inherited;
     fDown:=False;
     Color:=clBtnFace;
     Font.Color:=clBtnText;
     offset:=0;
     scrollLeftVisible:=FALSE;
     scrollRightVisible:=FALSE;
     scrollLeftHilite := FALSE;
     scrollRightHilite := FALSE;
     hilited := FALSE;
     fOldPath := '';
     pathWidth := 0;
     scrollTimerActive := FALSE;
End;

procedure TCoolDirectoryBar.queryChangePath;
var
   i: integer;
   p: string;
begin
     if assigned(fOnQueryChangePath) then begin
        p:='';
        for i:=0 to index do begin
            p := p + pathItems[i].folderName;
        end;
        fOnQueryChangePath(self, p);
     end;
end;

procedure TCoolDirectoryBar.WMMouseMove;
var
   x, y, i: integer;
   changed: boolean;
   res: integer;
begin
     x := LOWORD(message.LParam);
     y := HIWORD(message.LParam);

     changed := FALSE;
     res := mouseOverScrollBtn(x, y);
     if res <> NO_SCROLLER then begin
        if res = LEFT_SCROLLER then begin
           if not ScrollLeftHilite then changed := TRUE;
           ScrollLeftHilite := TRUE;
        end;
        if res = RIGHT_SCROLLER then begin
           if not ScrollRightHilite then changed := TRUE;
           ScrollRightHilite := TRUE;
        end;
        if pathItemsCount > 0 then begin
           for i:=0 to pathItemsCount - 1 do begin
               pathItems[i].hilite := FALSE;
           end;
        end;
     end else begin
        if ScrollLeftHilite or ScrollRightHilite then begin
           ScrollLeftHilite := FALSE;
           ScrollRightHilite := FALSE;
           changed := TRUE;
        end;
     end;
     if (res = NO_SCROLLER) and (pathItemsCount > 0) then begin
        for i:=0 to pathItemsCount - 1 do begin
            if not pathItems[i].staticText then begin
               if ((pathItems[i].left - offset) <= x) and (x < (pathItems[i].left + pathItems[i].width - offset)) then begin
                  if not pathItems[i].hilite then changed := TRUE;
                  pathItems[i].hilite := TRUE;
               end else begin
                  if pathItems[i].hilite then changed := TRUE;
                  pathItems[i].hilite := FALSE;
               end;
            end;
        end;
     end;

     if changed then Paint;
end;

procedure TCoolDirectoryBar.paint;
var
   i: integer;
   scr_x, scr_y: integer;

procedure draw3DLine(x1, y1, x2, y2: integer; sunken: boolean);
begin
     with self.Canvas do begin
          if sunken then begin
             pen.Color := clGray;
             MoveTo(x1, y2);
             LineTo(x1, y1);
             LineTo(x2, y1);
             pen.Color := clWhite;
             LineTo(x2, y2);
             LineTo(x1, y2);
          end else begin
             pen.Color := clWhite;
             MoveTo(x1, y2);
             LineTo(x1, y1);
             LineTo(x2, y1);
             pen.Color := clGray;
             LineTo(x2, y2);
             LineTo(x1, y2);
          end;
     end;
end;

begin
     with self.Canvas do begin

          self.Canvas.TextFlags := 0;
          brush.color := color;
          fillRect(Rect(0, 0, width, height));

          if pathItemsCount > 0 then begin
             for i:=0 to pathItemsCount - 1 do begin

                 if pathItems[i].hilite then
                    Font.Color := clBlue
                 else if hilited then
                    Font.Color := clWhite
                 else if pathItems[i].grayed then
                    Font.Color := clGray
                 else
                    Font.Color := clBlack;

                 if pathItems[i].active then
                    Font.Style := [fsBold]
                 else
                    Font.Style := [];

                 TextOut(pathItems[i].left - offset, pathItems[i].top, pathItems[i].folderName);
             end;
          end;

          if down then begin
             pen.Color := clGray;
             MoveTo(0, height-1);
             LineTo(0, 0);
             LineTo(width-1, 0);
             pen.Color := clWhite;
             LineTo(width-1, height-1);
             LineTo(0, height-1);
          end;

          scrollLeftVisible := (offset <> 0);
          scrollRightVisible := ((pathWidth - offset) > width);

          if scrollLeftVisible then begin
             if scrollLeftHilite then brush.Color := clGray else brush.color := clBtnFace;
             fillRect(Rect(0 + 1, 0 + 1, 0 + 1 + scrollerWidth, height - 1));
             brush.Color := clBlack;
             pen.color := clBlack;
             scr_x := 0 + 1 + ((scrollerWidth - 6) div 2);
             scr_y := 0 + 1 + ((height - 10) div 2);
             polygon([point(scr_x+4, scr_y), point(scr_x+4, scr_y+8), point(scr_x, scr_y+4)]);
             draw3dLine(0 + 1, 0 + 1, 0 + 1 + scrollerWidth, height - 1, false);
          end;
          if scrollRightVisible then begin
             if scrollRightHilite then brush.Color := clGray else brush.color := clBtnFace;
             fillRect(Rect(width - 1 - scrollerWidth, 0 + 1, width - 2, height - 1));
             brush.Color := clBlack;
             pen.color := clBlack;
             scr_x := width - 1 + ((scrollerWidth-2) div 2) - scrollerWidth;
             scr_y := 0 + 1 + ((height - 10) div 2);
             polygon([point(scr_x, scr_y), point(scr_x+4, scr_y+4), point(scr_x, scr_y+8)]);
             draw3dLine(width - 1 - scrollerWidth, 0 + 1, width - 2, height - 1, false);
          end;
     end;
end;

procedure TCoolDirectoryBar.SetPath;
var
   i: integer;
   tmp, tmp2: string;
   offset: integer;
   grayIt: boolean;
   activeIndex: integer;
begin
     activeIndex := 0;
     if path <> '' then begin
        if path[1] = '%' then begin
           pathItemsCount := 1;
           with pathItems[0] do begin
                folderName := copy(path, 2, length(path)-1);
                Width := self.Canvas.TextWidth(folderName);
                Height := self.Canvas.TextHeight(folderName);
                Top := 1;
                Left := 5;
                hilite := FALSE;
                active := FALSE;
                staticText := TRUE;
                grayed := FALSE;
           end;
           fPath := path;
           fOldpath := '';
        end else begin
           path := trim(path);
           if path[length(path)] <> '\' then path := path + '\';
           fPath := path;
           if fOldPath <> '' then begin
              if pos(path, fOldPath) = 1 then begin
                 path := fOldPath;
              end else
                 fOldPath := path;
           end else fOldPath := path;
           pathItemsCount := 0;
           tmp := '';
           offset := 5;
           grayIt := FALSE;
           tmp2 := '';
           for i := 1 to length(path) do begin
               tmp := tmp + path[i];
               if path[i] = '\' then begin
                  if i > length(fPath) then begin
                     grayIt := TRUE;
                  end;
                  with pathItems[pathItemsCount] do begin
                       folderName := copy(tmp, 1, length(tmp)-1);
                       Height := self.Canvas.TextHeight(folderName);
                       left := offset;
                       top := 1;
                       hilite := FALSE;
                       tmp2 := tmp2 + tmp;
                       if tmp2 = fPath then begin
                          activeIndex := pathItemsCount + 1;
                          active := TRUE;
                          self.Canvas.Font.Style := [fsBold];
                          Width := self.Canvas.TextWidth(folderName);
                          self.Canvas.Font.Style := [];
                       end else begin
                          Width := self.Canvas.TextWidth(folderName);
                          active := FALSE;
                       end;
                       inc(offset, Width);
                       staticText := FALSE;
                       grayed := grayIt;
                  end;
                  inc(pathItemsCount);
                  with pathItems[pathItemsCount] do begin
                       folderName := '\';
                       Width := self.Canvas.TextWidth(folderName);
                       Height := self.Canvas.TextHeight(folderName);
                       left := offset;
                       inc(offset, Width);
                       top := 1;
                       hilite := FALSE;
                       active := FALSE;
                       staticText := TRUE;
                       grayed := grayIt;
                  end;
                  inc(pathItemsCount);
                  tmp := '';
               end;
           end;
           pathWidth := offset;
        end;
     end else begin
        fPath:='';
        pathItemsCount := 0;
     end;
     self.offset := 0;
     scrollToItem(activeIndex, true);
end;

function TCoolDirectoryBar.GetPath;
begin
     result := fPath;
end;

Procedure TCoolDirectoryBar.CMMouseEnter;
Begin
     Inherited;
     If Down Then Begin
        Color:=clBtnFace;
        Canvas.Font.Color:=clBlack;
        hilited := FALSE;
     End Else Begin
        Color:=clBtnShadow;
        Canvas.Font.Color:=clWhite;
        hilited := TRUE;
     End;
     Paint;
End;

Procedure TCoolDirectoryBar.CMMouseLeave;
var
   i: integer;
Begin
     Inherited;
     If Down Then Begin
        Color:=cl3DLight;
        Canvas.Font.Color:=clBlack;
        hilited := FALSE;
     End Else Begin
        Color:=clBtnFace;
        Canvas.Font.Color:=clBtnText;
        hilited := FALSE;
     End;
     for i:=0 to pathItemsCount - 1 do begin
         pathItems[i].hilite := FALSE;
     end;
     ScrollLeftHilite := FALSE;
     ScrollRightHilite := FALSE;
     killTimer;
     Paint;
End;

Procedure TCoolDirectoryBar.SetDown;
Begin
     fDown:=Value;
     If Down Then Begin
        Color:=cl3DLight;
        Canvas.Font.Color:=clBlack;
        hilited := FALSE;
     End Else Begin
        Color:=clBtnFace;
        Canvas.Font.Color:=clBlack;
     End;
     Paint;
End;


end.
