unit finderpopup;

interface

Uses
    classes, menus, shlobj, activex, shellapi, windows, dialogs, sysutils, forms, controls, imglist, graphics;

const
   FINDER_BROWSE = 1;
   FINDER_SPECIALFOLDERS = 2;
   FINDER_LEFT = 4;
   FINDER_RIGHT = 8;
   FINDER_PATHLIST = 16;
   FINDER_ALL = FINDER_BROWSE + FINDER_LEFT + FINDER_RIGHT + FINDER_SPECIALFOLDERS + FINDER_PATHLIST;

type
    TNotifyObject = class
    public
          tag: integer;
          procedure clickHandler(sender: TObject);
    end;

function popupFinder(x, y, flags: integer; aHandle: hWnd; alignRight: boolean; initialDir: string): string;
function preparePathForMenu(const s: string): string;
function BrowseFolder(const InitialDir: string; var SelectedDir: string; aHandle: hWnd): Boolean;


implementation

uses
    mainform;

const
   maxSpecialFolders = 7;
   specialFolderConst: array[0..maxSpecialFolders] of integer =
                    (CSIDL_DESKTOPDIRECTORY,
                     CSIDL_PERSONAL,
                     CSIDL_PROGRAMS,
                     CSIDL_STARTMENU,
                     CSIDL_STARTUP,
                     CSIDL_RECENT,
                     CSIDL_TEMPLATES,
                     CSIDL_BITBUCKET
                    );
   specialFolderNames: array[0..maxSpecialFolders] of string =
                    ('Desktop',
                     'Eigene Dateien',
                     'Programme',
                     'Startmen',
                     'Autostart',
                     'Zuletzt verwendete Dokumente',
                     'Vorlagen',
                     'Papierkorb'
                    );

var
   pop: TPopupMenu;
   notifyObject: TNotifyObject;
   clickHandled: boolean;
   folders: TStringList;
   menuImageList: TImageList;
   desktop: PItemIDList;
   sysImageListSmall: THandle;
   FileInfo: TSHFileInfo;


function BrowseFolder(const InitialDir: string; var SelectedDir: string; aHandle: hWnd): Boolean;
var
  SelectedPIDL: PItemIDList;
  Malloc: IMalloc;
  BrowseInfo: TBrowseInfo;
  DisplayName: PChar;

  function BrowseCallbackProc(Wnd: THandle; Msg: UINT; lp: LParam;
    Data: LParam): Integer; stdcall;
  begin
    Result := 0;
    if (Data <> 0) and (Msg = BFFM_INITIALIZED) then
      SendMessage(Wnd, BFFM_SETSELECTION, 1, Data);
  end;

begin
  SHGetMalloc(Malloc);
  DisplayName := Malloc.Alloc(MAX_PATH);
  FillChar(BrowseInfo, sizeof(TBrowseInfo), 0);
  BrowseInfo.hwndOwner := Application.MainForm.Handle;
  BrowseInfo.pszDisplayName := DisplayName;
  if InitialDir <> '' then
  begin
    BrowseInfo.lpfn := TFNBFFCallback(@BrowseCallbackProc);
    BrowseInfo.lParam := Integer(PChar(InitialDir));
  end;
  BrowseInfo.lpszTitle := 'Whlen Sie einen Ordner...';
  BrowseInfo.ulFlags := BIF_RETURNONLYFSDIRS;
  BrowseInfo.hwndOwner := aHandle;
  SelectedPIDL := SHBrowseForFolder(BrowseInfo);
  Result := SelectedPIDL <> nil;
  if Result then
  begin
    SHGetPathFromIDList(SelectedPIDL, DisplayName);
    Malloc.Free(SelectedPIDL);
    SelectedDir := string(DisplayName);
  end;
  Malloc.Free(DisplayName);
end;

procedure TNotifyObject.clickHandler;
begin
     tag := (sender as tmenuItem).Tag;
     clickHandled := TRUE;
end;

function addBackslash(const s: string): string;
begin
     result := trim(s);
     if result <> '' then
        if result[length(result)] <> '\' then result := result + '\';
end;

function preparePathForMenu(const s: string): string;
var
   i: integer;
begin
     result := '';
     if s <> '' then begin
        for i := 1 to length(s) do begin
            result := result + s[i];
            if result[length(result)] = '&' then begin
               result := result + '&';
            end;
        end;
     end;
end;

function popupFinder;
var
   newitem: TMenuItem;
   i: integer;
   buffer: array[0..MAX_PATH] of char;
   pidl: PItemIDList;
   dummy: string;
begin
     with pop do begin
          items.Clear;
          folders.Clear;
          if (flags and FINDER_BROWSE) <> 0 then begin
             newitem := TMenuItem.Create(pop);
             newitem.Caption := 'Durchsuchen...';
             newitem.OnClick := notifyObject.clickHandler;
             newitem.Tag := 1;
             newitem.ImageIndex := -1;
             items.Add(newitem);
             newitem := TMenuItem.Create(pop);
             newitem.Caption := '-';
             newitem.Tag := 0;
             items.Add(newitem);
          end;
          if ((flags and FINDER_LEFT) <> 0) or ((flags and FINDER_RIGHT) <> 0) then begin
             if (flags and FINDER_LEFT) <> 0 then begin
                newitem := TMenuItem.Create(pop);
                newitem.Caption := 'Links - '+data[LeftList].Directory+'\';
                newitem.OnClick := notifyObject.clickHandler;
                newitem.tag := folders.Add(data[LeftList].Directory+'\') + 100;
                newitem.ImageIndex := getMenuIconIndex(data[LeftList].Directory+'\');
                items.Add(newitem);
             end;
             if (flags and FINDER_RIGHT) <> 0 then begin
                newitem := TMenuItem.Create(pop);
                newitem.Caption := 'Rechts - '+data[RightList].Directory+'\';
                newitem.OnClick := notifyObject.clickHandler;
                newitem.tag := folders.Add(data[RightList].Directory+'\') + 100;
                newitem.ImageIndex := getMenuIconIndex(data[RightList].Directory+'\');
                items.Add(newitem);
             end;
             newitem := TMenuItem.Create(pop);
             newitem.Caption := '-';
             newitem.Tag := 0;
             items.Add(newitem);
          end;
          if (flags and FINDER_SPECIALFOLDERS) <> 0 then begin
             for i := 0 to maxSpecialFolders do begin
                 if succeeded(SHGetSpecialFolderLocation(FrmMain.Handle, SpecialFolderConst[i], pidl)) then begin
                    if SHGetPathFromIDList(pidl, buffer) then begin
                       newitem := TMenuItem.Create(pop);
                       newitem.Caption := specialFolderNames[i];
                       newitem.OnClick := notifyObject.clickHandler;
                       newitem.Tag := folders.Add(string(buffer)) + 100;
                       newitem.ImageIndex := getMenuIconIndex(string(buffer));
                       items.Add(newitem);
                    end;
                 end;
             end;
             newitem := TMenuItem.Create(pop);
             newitem.Caption := '-';
             newitem.Tag := 0;
             items.Add(newitem);
          end;
          if ((flags and FINDER_PATHLIST) <> 0) and (FrmMain.PathNum > 0) then begin
             for i := 0 to FrmMain.PathNum - 1 do begin
                 newitem := TMenuItem.Create(pop);
                 newitem.Caption := preparePathForMenu(addBackslash(FrmMain.Paths[i].Path));
                 newitem.Tag := folders.Add(addBackslash(FrmMain.Paths[i].Path)) + 100;
                 newitem.OnClick := notifyObject.clickHandler;
                 newitem.ImageIndex := getMenuIconIndex(addBackslash(FrmMain.Paths[i].Path));
                 items.add(newitem);
             end;
          end;
     end;
     result := '';
     notifyObject.tag := 0;
     clickHandled := FALSE;

     if alignRight then begin
        pop.Alignment := paRight;
     end else begin
        pop.Alignment := paLeft;
     end;

     pop.Popup(x, y);
     application.ProcessMessages;
     if clickHandled then begin
        if notifyObject.tag = 1 then begin
           if BrowseFolder(initialDir, dummy, aHandle) then
              result := dummy
           else
              result := '';
        end else if notifyObject.tag = 0 then begin
           result := '';
        end else begin
           if ((notifyObject.tag - 100) >= 0) and ((notifyObject.tag - 100) < folders.Count) then begin
              result := folders[notifyObject.tag - 100];
           end else result := '';
        end;
     end else begin
        result := '';
     end;

end;

initialization
   SHGetSpecialFolderLocation(INVALID_HANDLE_VALUE, CSIDL_DESKTOP, desktop);
   SysImageListSmall:=SHGetFileInfo(PChar(desktop), 0, FileInfo, SizeOf(FileInfo), SHGFI_PIDL Or SHGFI_SYSICONINDEX Or SHGFI_SMALLICON);

   MenuImageList := TImageList.Create(FrmMain);
   MenuImageList.ShareImages := True;
   MenuImageList.BkColor := clNone;
   MenuImageList.Handle := SysImageListSmall;
   MenuImageList.DrawingStyle := dsTransparent;

   notifyObject := TNotifyObject.Create;
   folders := TStringList.Create;
   pop := TPopupMenu.Create(frmMain);
   pop.Images := MenuImageList;
finalization
   pop.Free;
   notifyObject.Free;
   folders.Free;
end.
