unit pidl;

interface

uses
    shlobj, activex,shellApi, windows;

function CreatePIDL(Size: Integer): PItemIDList;
function NextPIDL(IDList: PItemIDList): PItemIDList;
function StripLastID(IDList: PItemIDList): PItemIDList;
function GetItemCount(IDList: PItemIDList): Integer;
function GetPIDLSize(IDList: PItemIDList): Integer;
function CopyPIDL(IDList: PItemIDList): PItemIDList;
function ConcatPIDLs(IDList1, IDList2: PItemIDList): PItemIDList;
function RelativeFromAbsolute(AbsoluteID: PItemIDList): PItemIDList;
procedure freePIDL(var PIDL: PItemIDList);

implementation

var
   malloc: IMalloc;

procedure freePIDL(var PIDL: PItemIDList);
begin
     if assigned(PIDL) then begin
        malloc.Free(PIDL);
        PIDL := NIL;
     end;
end;

function CreatePIDL(Size: Integer): PItemIDList;
begin
  Result := Malloc.Alloc(Size);
  if Assigned(Result) then
    FillChar(Result^, Size, 0);
end;

function NextPIDL(IDList: PItemIDList): PItemIDList;
begin
  Result := IDList;
  Inc(PChar(Result), IDList^.mkid.cb);
end;

function StripLastID(IDList: PItemIDList): PItemIDList;
var
  MarkerID: PItemIDList;
  oldCB: word;
  dummy: PItemIDList;
begin
  result := NIL;
  MarkerID := IDList;
  dummy := IDList;
  if Assigned(IDList) then
  begin
    while dummy.mkid.cb <> 0 do
    begin
      MarkerID := dummy;
      dummy := NextPIDL(dummy);
    end;
    oldCB := MarkerID.mkid.cb;
    MarkerID.mkid.cb := 0;
    result := copyPIDL(IDList);
    MarkerID.mkid.cb := oldCB;
  end;
end;

function GetItemCount(IDList: PItemIDList): Integer;
begin
  Result := 0;
  while IDList^.mkid.cb <> 0 do
  begin
    Inc(Result);
    IDList := NextPIDL(IDList);
  end;
end;

function GetPIDLSize(IDList: PItemIDList): Integer;
begin
  Result := 0;
  if Assigned(IDList) then
  begin
    Result := SizeOf(IDList^.mkid.cb);
    while IDList^.mkid.cb <> 0 do
    begin
      Result := Result + IDList^.mkid.cb;
      IDList := NextPIDL(IDList);
    end;
  end;
end;

function CopyPIDL(IDList: PItemIDList): PItemIDList;
var
  Size: Integer;
begin
  Size := GetPIDLSize(IDList);
  Result := CreatePIDL(Size);
  if Assigned(Result) then
    CopyMemory(Result, IDList, Size);
end;

function ConcatPIDLs(IDList1, IDList2: PItemIDList): PItemIDList;
var
  cb1, cb2: Integer;
begin
  if Assigned(IDList1) then
    cb1 := GetPIDLSize(IDList1) - SizeOf(IDList1^.mkid.cb)
  else
    cb1 := 0;

  cb2 := GetPIDLSize(IDList2);

  Result := CreatePIDL(cb1 + cb2);
  if Assigned(Result) then
  begin
    if Assigned(IDList1) then
      CopyMemory(Result, IDList1, cb1);
    CopyMemory(PChar(Result) + cb1, IDList2, cb2);
  end;
end;

function RelativeFromAbsolute(AbsoluteID: PItemIDList): PItemIDList;
begin
  Result := AbsoluteID;
  while GetItemCount(Result) > 1 do
     Result := NextPIDL(Result);
  Result := CopyPIDL(Result);
end;

initialization
    SHGetMalloc(malloc);
finalization
    malloc := NIL;
end.
